/*! react-deep-diff v0.1.1 | MIT License */
import React from 'react';
import { deepdiff, DiffOperation } from '@gera2ld/deepdiff';
import { render, ItemTypes } from '@gera2ld/format-json';
import Prism from 'prismjs/components/prism-core';
import 'prismjs/components/prism-json';
import 'prismjs/themes/prism-tomorrow.css';

var styles = {"diff":"dd-diff__9ETNF","removed":"dd-removed__1FiYE","added":"dd-added__2S-D9","empty":"dd-empty__1FIq0","line":"dd-line__3kVyO","marker":"dd-marker__2m1qU","content":"dd-content__1MvKK"};

function _createForOfIteratorHelperLoose(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (it) return (it = it.call(o)).next.bind(it); if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }
var REPLACE_SUFFIX = '::replace';
var PATH_START = 'pathStart';
var PATH_END = 'pathEnd';

function compare(a, b) {
  if (a < b) return -1;
  if (a > b) return 1;
  return 0;
}

var jsonFormatOptions = {
  indent: 2,
  quoteAsNeeded: false,
  quote: '"',
  trailing: false,
  template: false,
  entries: function entries(data) {
    return Object.entries(data).map(function (_ref) {
      var key = _ref[0],
          value = _ref[1];
      return [key, value, key.endsWith(REPLACE_SUFFIX) ? key.slice(0, -REPLACE_SUFFIX.length) : key];
    }).sort(function (a, b) {
      return compare(a[2], b[2]) || compare(a[0], b[0]);
    });
  },
  onData: function onData(item) {
    item.data = [{
      type: PATH_START,
      value: '',
      path: item.path
    }].concat(item.data, [{
      type: PATH_END,
      value: '',
      path: item.path
    }]);
  }
};

function getLineInfo(root) {
  var lines = [];
  var line = [];
  var pathMap = {};
  root.data.forEach(function (item) {
    if (item.type === PATH_START) {
      pathMap[item.path.join('/')] = [lines.length];
    } else if (item.type === PATH_END) {
      pathMap[item.path.join('/')].push(lines.length);
    } else if (item.type === ItemTypes.BR) {
      lines.push(line.join(''));
      line = [];
    } else {
      line.push(item.value);
    }
  });
  lines.push(line.join(''));
  return {
    lines: lines,
    pathMap: pathMap
  };
}

function DeepDiff(_ref2) {
  var obj1 = _ref2.obj1,
      obj2 = _ref2.obj2,
      className = _ref2.className;
  if (!obj1 || !obj2) return null;
  var diff = deepdiff(obj1, obj2);
  var mergedObj = JSON.parse(JSON.stringify(obj1));
  var insertPaths = [];
  var removePaths = [];
  var replacePaths = [];
  diff.forEach(function (item) {
    var keys = item.path.slice(1).split('/');
    var parentPath = keys.slice(0, -1);
    var parent = parentPath.reduce(function (a, b) {
      return a[b];
    }, mergedObj);

    if (item.op === DiffOperation.INSERT || item.op === DiffOperation.REPLACE) {
      var insertKeys = [].concat(keys);
      var last = keys[keys.length - 1];

      if (Array.isArray(parent)) {
        var offset = parent.offset || 0;
        parent.splice(offset + +last, 0, item.newVal);
        insertKeys[insertKeys.length - 1] = +last + offset;
        parent.offset = offset + 1;
      } else {
        if (item.op === DiffOperation.REPLACE) {
          last += REPLACE_SUFFIX;
          insertKeys[insertKeys.length - 1] = last;
        }

        parent[last] = item.newVal;
      }

      if (item.op === DiffOperation.INSERT) insertPaths.push(insertKeys);
    }

    if (item.op === DiffOperation.DELETE || item.op === DiffOperation.REPLACE) {
      var removeKeys = [].concat(keys);

      if (Array.isArray(parent)) {
        removeKeys.push(+removeKeys.pop() + (parent.offset || 0));
      }

      if (item.op === DiffOperation.DELETE) removePaths.push(removeKeys);
    }

    if (item.op === DiffOperation.REPLACE) replacePaths.push(keys);
  });
  var root = render(mergedObj, jsonFormatOptions);
  var lineInfo = getLineInfo(root);
  var types = lineInfo.lines.map(function () {
    return 0;
  });

  var updateTypes = function updateTypes(paths, v) {
    for (var _iterator = _createForOfIteratorHelperLoose(paths), _step; !(_step = _iterator()).done;) {
      var keys = _step.value;
      var path = keys.join('/');
      var pos = lineInfo.pathMap[path];

      if (pos) {
        var start = pos[0],
            end = pos[1];

        for (var i = start; i <= end; i += 1) {
          types[i] = v;
        }
      }
    }
  };

  updateTypes(insertPaths, 1);
  updateTypes(removePaths, 2);
  updateTypes(replacePaths, 3);
  updateTypes(replacePaths.map(function (keys) {
    return [].concat(keys.slice(0, -1), [keys[keys.length - 1] + REPLACE_SUFFIX]);
  }), 4);
  var blocks = [];
  var cur;

  for (var i = 0; i < lineInfo.lines.length; i += 1) {
    var _cur;

    var line = Prism.highlight(lineInfo.lines[i], Prism.languages.json, 'json5');
    var type = types[i]; // merge type 3 and 4 as REPLACE

    if (type === 4) type = 3;

    if (((_cur = cur) == null ? void 0 : _cur.type) !== type) {
      cur = {
        type: type,
        left: [],
        right: []
      };
      blocks.push(cur);
    } // switch by original type


    switch (types[i]) {
      case 1:
        // insert
        cur.right.push(line);
        break;

      case 2:
        // delete
        cur.left.push(line);
        break;

      case 3:
        // replace from
        cur.left.push(line);
        break;

      case 4:
        // replace to
        cur.right.push(line);
        break;

      default:
        cur.left.push(line);
        cur.right.push(line);
    }
  }

  return /*#__PURE__*/React.createElement("div", {
    className: styles.diff + " " + (className || '')
  }, blocks.map(function (item, i) {
    return Array.from({
      length: Math.max(item.left.length, item.right.length)
    }, function (_, j) {
      var classLeft = [item.type > 1 && item.left[j] && styles.removed, item.type === 1 && styles.empty].filter(Boolean).join(' ');
      var classRight = [item.type % 2 > 0 && item.right[j] && styles.added, item.type === 2 && styles.empty].filter(Boolean).join(' ');
      return /*#__PURE__*/React.createElement("div", {
        key: i + "-" + j,
        className: styles.line
      }, /*#__PURE__*/React.createElement("div", {
        className: styles.marker + " " + classLeft
      }), /*#__PURE__*/React.createElement("div", {
        className: styles.content + " " + classLeft,
        dangerouslySetInnerHTML: {
          __html: item.left[j]
        }
      }), /*#__PURE__*/React.createElement("div", {
        className: styles.marker + " " + classRight
      }), /*#__PURE__*/React.createElement("div", {
        className: styles.content + " " + classRight,
        dangerouslySetInnerHTML: {
          __html: item.right[j]
        }
      }));
    });
  }));
}
DeepDiff.defaultProps = {
  className: ''
};

export { DeepDiff };
